//Conventions:
//    Global constants (declared with const) and #defines - all uppercase letters with words separated 
//        by underscores.
//        (E.G., #define MY_DEFINE 5).
//        (E.G., const int MY_CONSTANT = 5;).
//    New data types (classes, structs, typedefs, etc.) - begin with an uppercase letter followed by
//        lowercase words separated by uppercase letters.  Enumerated constants contain a prefix
//        associating them with a particular enumerated set.
//        (E.G., typedef int MyTypedef;).
//        (E.G., enum MyEnumConst {MEC_ONE, MEC_TWO};)
//    Global variables - begin with "g_" followed by lowercase words separated by underscores.
//        (E.G., int g_my_global;).
//    Local variables - begin with a lowercase letter followed by lowercase words separated by
//        underscores.
//        (E.G., int my_local;).
//    Argument variables - begin with "a_" followed by lowercase words separated by underscores.
//        (E.G., ...int a_my_argument, ...).
//    Member variables - begin with "m_" followed by lowercase words separated by underscores.
//        (E.G., int m_my_member;).
//    Functions (member or global) - begin with an uppercase letter followed by lowercase words
//        separated by uppercase letters.
//        (E.G., void MyFunction(void);).


//******************************************************************************************************
//**** PROJECT HEADER FILES
//******************************************************************************************************
#include "ImageDump.h"
#include "PlainBufferedFile.h"		//ImageDump requires my Santa's gift bag distribution, which is
									//available on BeWare, or at http://www-biology.ucsd.edu/~btietz/BeOS-software.html
#include "Colors.h"


//******************************************************************************************************
//**** APPLICATION
//******************************************************************************************************
int main()
{
	new ImageDumpApp;
	be_app->Run();
	delete be_app;
}


ImageDumpApp::ImageDumpApp()
: DocumentApp("application/x-vnd.BT-ImageDump")
{
	m_doc_title_number = 1;

	//Set up the open file panel
	m_open_panel = NULL;
	status_t error;
	BMessenger app_messenger((BHandler*)NULL,this,&error);
	if(error == B_OK)
		m_open_panel = new BFilePanel(B_OPEN_PANEL,&app_messenger,NULL);
}


ImageDumpApp::~ImageDumpApp()
{ }



void ImageDumpApp::OpenNewDocument()
{
	char name_buffer[13];
	sprintf(name_buffer,"Untitled %d",atomic_add(&m_doc_title_number,1));
	new BitmapImageWindow(m_window_position_set.GetNextUntitledWindow(),name_buffer);
}


void ImageDumpApp::MessageReceived(BMessage* a_message)
{
	switch(a_message->what)
	{
		case MSG_NEW_DOCUMENT:
			OpenNewDocument();
			break;
		case MSG_OPEN_DOCUMENT:
			m_open_panel->Show();
			break;
		default:
			DocumentApp::MessageReceived(a_message);
			break;
	}
}


void ImageDumpApp::RefsReceived(BMessage* a_message)
{
	uint32 type;
	int32 count;
	entry_ref file_entry_ref;

	a_message->GetInfo("refs", &type, &count);
	if(type != B_REF_TYPE)
		return;
	for(uint32 ref_index = 0; ref_index < count; ref_index++)
	{
		if(a_message->FindRef("refs", ref_index, &file_entry_ref) == B_OK)
		{
			BEntry file_entry(&file_entry_ref,true);
			if(file_entry.InitCheck() == B_NO_ERROR)
			{
				BPath file_path;
				if(file_entry.GetPath(&file_path) == B_NO_ERROR)
					new BitmapImageWindow(m_window_position_set.GetNextUntitledWindow(),file_path.Path());
			}
		}
	}
}


void ImageDumpApp::ArgvReceived(int32 argc, char **argv)
{
	if(argc > 1)
		for(int32 counter = 1; counter < argc; counter++)
			new BitmapImageWindow(m_window_position_set.GetNextUntitledWindow(),argv[counter]);
}


void ImageDumpApp::ReadyToRun()
{
	if(CountRegWindProcHolds() == 0)
	{
		BAlert *alert = new BAlert(NULL,"Usage: Drop an image onto the ImageDump application or "
			"pass the image filename as an argument at the command line\n","OK",NULL,NULL);
		alert->Go();
		PostMessage(B_QUIT_REQUESTED);
	}
}


//******************************************************************************************************
//**** WINDOW
//******************************************************************************************************
BitmapImageWindow::BitmapImageWindow(BRect a_frame,const char* a_filename)
: FileDocumentWindow(a_frame,a_filename,B_TITLED_WINDOW,B_WILL_ACCEPT_FIRST_CLICK)
{
	BMenuBar* menu_bar = GenerateMenuBar();
	AddChild(menu_bar);

	BRect r = Bounds();
	r.top = menu_bar->Frame().bottom+1.0;
	m_image_view = new ImageView(r);
	m_image_view->SetBitmap(BTranslationUtils::GetBitmapFile(a_filename));
	if(m_image_view->GetBitmap())
	{
		if(m_image_view->GetBitmap()->ColorSpace() == B_CMAP8)
		{
			item_CMAP8->SetMarked(true);
			item_RGB32->SetMarked(false);
		}
		else
		{
			item_CMAP8->SetMarked(false);
			item_RGB32->SetMarked(true);
		}
	}
	else
	{
		item_CMAP8->SetMarked(false);
		item_RGB32->SetMarked(false);
	}	
	AddChild(m_image_view);

	char* new_filename = new char[strlen(a_filename)+3];
	strcpy(new_filename,a_filename);
	strcat(new_filename,".h");
	SetFileName(new_filename);
	delete[] new_filename;

	Show();
}


BMenuBar* BitmapImageWindow::GenerateMenuBar()
{
	BMenuBar* menu_bar = new BMenuBar(BRect(0,0,0,0),"Menu");
	BMenu* menu;
	BMenu* submenu;
	BMenuItem* item;

	//File Menu
	menu = new BMenu("File");
		//Open...
			item = new BMenuItem("Open"B_UTF8_ELLIPSIS,new BMessage(MSG_OPEN_DOCUMENT),'O');
			item->SetTarget(be_app);
			menu->AddItem(item);
		menu->AddSeparatorItem();
		//Save as
			menu->AddItem(new BMenuItem("Save as source code"B_UTF8_ELLIPSIS,new BMessage(FDW_SAVE_AS_REQESTED),0));
		menu->AddSeparatorItem();
		//Colors
			submenu = new BMenu("Color space");
				item_CMAP8 = new BMenuItem("B_CMAP8",new BMessage(MSG_CMAP8),0);
				submenu->AddItem(item_CMAP8);
				item_RGB32 = new BMenuItem("B_RGB32",new BMessage(MSG_RGB32),0);
				submenu->AddItem(item_RGB32);
			menu->AddItem(new BMenuItem(submenu));
		menu->AddSeparatorItem();
		//Close window
			menu->AddItem( new BMenuItem("Close",new BMessage(B_QUIT_REQUESTED),'W'));
		//Quit
			item = new BMenuItem("Quit",new BMessage(B_QUIT_REQUESTED),'Q');
			item->SetTarget(be_app);
			menu->AddItem(item);
	menu_bar->AddItem(menu);
	return menu_bar;
}


BitmapImageWindow::~BitmapImageWindow()
{ }


void BitmapImageWindow::MessageReceived(BMessage* a_message)
{
	const BBitmap* bitmap;
	BBitmap* new_bitmap;
	switch(a_message->what)
	{
		case MSG_CMAP8:
			bitmap = m_image_view->GetBitmap();
			if(bitmap && bitmap->ColorSpace() != B_CMAP8)
			{
				BBitmap* temp_bitmap = new BBitmap(bitmap->Bounds(),B_CMAP8,true);
				BView* temp_view = new BView(bitmap->Bounds(),NULL,B_FOLLOW_ALL_SIDES,0);
				temp_bitmap->AddChild(temp_view);
				temp_view->Window()->Lock();
				temp_view->DrawBitmap(bitmap,bitmap->Bounds());
				temp_view->Window()->Unlock();

				new_bitmap = new BBitmap(bitmap->Bounds(),B_CMAP8);
				int32* oldbits = (int32*)temp_bitmap->Bits();
				int32* newbits = (int32*)new_bitmap->Bits();
				int32 copycount = temp_bitmap->BitsLength()/4;
				for(int32 x = 0; x < copycount; x++)
					newbits[x] = oldbits[x];
				delete temp_bitmap;
				m_image_view->SetBitmap(new_bitmap);
				item_CMAP8->SetMarked(true);
				item_RGB32->SetMarked(false);
			}
			break;
		case MSG_RGB32:
			bitmap = m_image_view->GetBitmap();
			if(bitmap && bitmap->ColorSpace() != B_RGB32)
			{
				BBitmap* temp_bitmap = new BBitmap(bitmap->Bounds(),B_RGB32,true);
				BView* temp_view = new BView(bitmap->Bounds(),NULL,B_FOLLOW_ALL_SIDES,0);
				temp_bitmap->AddChild(temp_view);
				temp_view->Window()->Lock();
				temp_view->DrawBitmap(bitmap,bitmap->Bounds());
				temp_view->Window()->Unlock();

				new_bitmap = new BBitmap(bitmap->Bounds(),B_RGB32);
				int32* oldbits = (int32*)temp_bitmap->Bits();
				int32* newbits = (int32*)new_bitmap->Bits();
				int32 copycount = temp_bitmap->BitsLength()/4;
				for(int32 x = 0; x < copycount; x++)
					newbits[x] = oldbits[x];
				delete temp_bitmap;
				m_image_view->SetBitmap(new_bitmap);
				item_CMAP8->SetMarked(false);
				item_RGB32->SetMarked(true);
			}
			break;
		default:
			FileDocumentWindow::MessageReceived(a_message);
			break;
	}
}


bool BitmapImageWindow::Save()
{
	const BBitmap* bitmap = m_image_view->GetBitmap();
	if(bitmap == NULL)
		return true;

	const char* filename = GetFileName();

	PlainBufferedFile save_file(filename,B_WRITE_ONLY|B_CREATE_FILE|B_ERASE_FILE);
	if(save_file.InitCheck() != B_NO_ERROR)
		return false;

	const char* write_text1 = "#include \"PrefilledBitmap.h\"\n\n//********Icon for '";
	const char* temp = GetLeafName();
	int x;
	for(x = 0; x < strlen(temp); x++)
		if(temp[x] == '.')
			break;
	char* write_text2 = new char[x+1];
	strncpy(write_text2,temp,x);
	write_text2[x] = 0;
	const char* write_text3 = "'\nuint8 ";
	const char* write_text4 = write_text2;
	char write_text5[32];
	int32 bits_length = bitmap->BitsLength();
	color_space colors = bitmap->ColorSpace();
	if(colors == B_RGB32)
		bits_length = bits_length*3/4;
	sprintf(write_text5,"_MData[%d] =\n{\n	",bits_length);
	uint8* bits = (uint8*)bitmap->Bits();

	#define WRITE_TEXT(write_text); \
		if(save_file.BufferedWrite(write_text,strlen(write_text)) != strlen(write_text)) return false;

	WRITE_TEXT(write_text1);
	WRITE_TEXT(write_text2);
	WRITE_TEXT(write_text3);
	WRITE_TEXT(write_text4);
	WRITE_TEXT(write_text5);
	char text_buffer[26];
	int32 bitmap_pos = 0;
	for(int32 counter = 0; counter < bits_length; counter++)
	{
		if(colors == B_RGB32)
			bitmap_pos = (counter/3)*4 + 2-(counter%3);			//BGRA -> RGB
		else
			bitmap_pos = counter;
		if(counter == bits_length-1)
		{
			sprintf(text_buffer,"0x%02X\n};\nPrefilledBitmap *",bits[bitmap_pos]);
			WRITE_TEXT(text_buffer);
			WRITE_TEXT(write_text2);
			const char* write_text6 = "_MICN;\n\n//Put this after your BApplication object has been constructed:\n	"
			WRITE_TEXT(write_text6);
			WRITE_TEXT(write_text2);
			const char* write_text7 = "_MICN = new PrefilledBitmap(BRect(";
			WRITE_TEXT(write_text7);
			BRect bitmap_bounds = bitmap->Bounds();
			sprintf(text_buffer,"%.1f,%.1f,%.1f,%.1f),",bitmap_bounds.left,bitmap_bounds.top,bitmap_bounds.right,bitmap_bounds.bottom);
			WRITE_TEXT(text_buffer);
			const char* write_text8 = "B_CMAP8,";
			const char* write_text32 = "B_RGB32,";
			const char* write_text9 = "/*Other color space*/,";
			if(colors == B_CMAP8)
			{
				WRITE_TEXT(write_text8);
			}
			else if(colors == B_RGB32)
			{
				WRITE_TEXT(write_text32);
			}
			else
			{
				WRITE_TEXT(write_text9);
			}
			WRITE_TEXT(write_text2);
			const char* write_text10 = "_MData);\n";
			WRITE_TEXT(write_text10);
		}
		else if(counter%16 < 15)
		{
			sprintf(text_buffer,"0x%02X, ",bits[bitmap_pos]);
			WRITE_TEXT(text_buffer);
		}
		else
		{
			sprintf(text_buffer,"0x%02X,\n	",bits[bitmap_pos]);
			WRITE_TEXT(text_buffer);
		}
	}
	if(save_file.FlushBuffer() != B_NO_ERROR)
		return false;

	BNodeInfo node_info(&save_file);
	node_info.SetType("text/x-source-code");
	delete[] write_text2;
	return true;
}


//******************************************************************************************************
//**** IMAGE VIEW
//******************************************************************************************************
ImageView::ImageView(BRect a_frame)
: BView(a_frame,NULL,B_FOLLOW_ALL_SIDES,B_WILL_DRAW)
{
	SetViewColor(White);
	m_bitmap = NULL;
}


ImageView::~ImageView()
{
	if(m_bitmap)
		delete m_bitmap;
}


void ImageView::Draw(BRect a_update_rect)
{
	if(m_bitmap)
	{
		BRect bitmap_rect = m_bitmap->Bounds();
		bitmap_rect.OffsetBy(16,16);
		if(a_update_rect.Intersects(bitmap_rect))
			DrawBitmap(m_bitmap,bitmap_rect);
	}
}


void ImageView::SetBitmap(BBitmap* a_bitmap)
{
	if(m_bitmap)
		delete m_bitmap;
	m_bitmap = a_bitmap;
	if(m_bitmap)
	{
		if(m_bitmap->ColorSpace() != B_RGB32 && m_bitmap->ColorSpace() != B_CMAP8)
		{
			BBitmap* new_bitmap = new BBitmap(m_bitmap->Bounds(),B_RGB32);
			new_bitmap->SetBits(m_bitmap->Bits(),m_bitmap->BitsLength(),0,m_bitmap->ColorSpace());
			delete m_bitmap;
			m_bitmap = new_bitmap;
		}
	}
	Invalidate();
}

const BBitmap* ImageView::GetBitmap()
{
	return m_bitmap;
}


